# Project to automate Agisoft Process
# Automation of the floor alignment and setting the correct bounding box.
# Assembled/Written by: Richard Garsthagen - www.pi3dscan.com
#
# for instructions on how to use see: http://www.pi3dscan.com/index.php/instructions/item/agisoft-how-to-process-a-scan-with-projection
#

import PhotoScan
import math

doc = PhotoScan.app.document
chunk = doc.chunks[0]     # This points to the first chunk!

#   py
#   
#   |
#   |
#   |
#   
#   p0 ------------ px

##########################################
##                                      ##
##   Change your variables here         ##
##                                      ##
##########################################

p0 = "target 3"   
px = "target 4"
py = "target 1"

distancepx = 0.7
distancepy = 0.4

boxwidth = 0.8
boxdepth = 0.7
boxheight = 2.235

c1target = "target 21"
c2target = "target 22"
c3target = "target 6"
c4target = "target 7"

##########################################
##                                      ##
##   End of variables                   ##
##                                      ##
##########################################


mp0 = 0
mpy = 0
mpx = 0

fp0 = 0
fpy = 0
fpx = 0


#setting for Y up, Z forward -> needed for mixamo/unity

vector0 = PhotoScan.Vector((0,0,0))
vectorY = PhotoScan.Vector((0,0,distancepy))   # Specify Y Distance
vectorX = PhotoScan.Vector((distancepx,0,0))   # Specify X Distance

c1 = 0
c2 = 0
c3 = 0
c4 = 0
c = 0

for m in chunk.markers:
  if m.label == c1target:
    print ("Center 1 point found")
    c1 = c
  if m.label == c2target:
    print ("Center 2 point found")
    c2 = c
  if m.label == c3target:
    print ("Center 3 point found")
    c3 = c
  if m.label == c4target:
    print ("Center 4 point found")
    c4 = c
  if m.label == p0: 
    mp0 = c
    fp0 = 1
    m.reference.location = vector0
    m.reference.enabled = 1
    print ("Found center point")
  if m.label == py: 
    mpy = c
    fpy = 1
    m.reference.location = vectorY
    m.reference.enabled = 1
    print ("found Y point")
    
  if m.label == px: 
    mpx = c
    fpx = 1
    m.reference.location = vectorX
    m.reference.enabled = 1
    print ("found X point")
  c = c + 1



if fp0 and fpx and fpy:
  print ("Found all markers")
  chunk.updateTransform()
else:
  print ("Error: not all markers found")

newregion = chunk.region

T = chunk.transform.matrix
v_t = T * PhotoScan.Vector( [0,0,0,1] )
m = PhotoScan.Matrix().diag([1,1,1,1])	

m = m * T
s = math.sqrt(m[0,0] ** 2 + m[0,1] ** 2 + m[0,2] ** 2) #scale factor
R = PhotoScan.Matrix( [[m[0,0],m[0,1],m[0,2]], [m[1,0],m[1,1],m[1,2]], [m[2,0],m[2,1],m[2,2]]])
R = R * (1. / s)
newregion.rot = R.t()

# Calculate center point of the bounding box, by taking the average of 2 left and 2 right markers 
mx = (chunk.markers[c1].position + chunk.markers[c2].position + chunk.markers[c3].position + chunk.markers[c4].position) / 4

mx = PhotoScan.Vector([mx[0], mx[1], mx[2]])
newregion.center = mx

dist = chunk.markers[mp0].position - chunk.markers[mpy].position
dist = dist.norm()

ratio = dist / distancepy

newregion.size = PhotoScan.Vector([boxwidth* ratio, boxheight* ratio, boxdepth * ratio])

chunk.region = newregion
chunk.updateTransform()

print ("Bounding box should be aligned now")



  
  

